from crystally.core.lattice import *


def interpolate_lattices(lattice1: Lattice, lattice2: Lattice, weighting=0.5):
    """ Linearly interpolate the atom list two lattices and return new lattice. Mind that the two lattices need to be
    sorted first!

    :param lattice1: first :class:`~crystally.core.lattice.Lattice`
    :param lattice2: second :class:`~crystally.core.lattice.Lattice`
    :param weighting: weighting between the two lattices. With weighting=0 the first, with weighting=1.0 the second
    lattice is returned. With weighting=0.5 (default value) every new atom position lies exactly between the ones of
    the provided lattices.
    :return: new interpolated :class:`~crystally.core.lattice.Lattice`
    """
    if len(lattice1.atoms) != len(lattice2.atoms):
        raise ValueError("lattices must have the same number of atoms!")

    new_lattice = Lattice(vectors=lattice1.vectors, atoms=[])
    for atom1, atom2 in zip(lattice1, lattice2):
        new_position = atom1.position + atom1.position.diff(atom2.position)*weighting
        new_atom = Atom(element=atom1.element, position=new_position, sublattice=atom1.sublattice)
        new_lattice.atoms.append(new_atom)
    return new_lattice


def get_element_number_list(lattice):
    """Get a two dimensional list with the sort order of the elements within the lattice.

    The first column specifies the element name and the second column the number of adjacent atoms with this
    element. This information can be used for VASP Input files.
    For instance if the sort order of the atoms is as follows:

    X, X, Y, Y, Y, X, X

    (where X and Y are the elements of the atoms)
    the function would return the following table:

    =============  =============================
    Element Name   Number of repeating elements
    =============  =============================
    X              2
    Y              4
    X              2
    =============  =============================

    :return: two dimensional list - the first column is an int, the second a string

    Examples
    --------
    >>> #lattice = generate_from_crystal(ceria(),2,2,2).sort("position")
    >>> #print(lattice.get_element_number_list())
    [['Ce', 1], ['O', 1], ['Ce', 3], ['O', 7]]
    """
    element_number_list = []
    for atom in lattice.atoms:
        if not element_number_list:
            element_number_list.append([atom.element, 1])
        elif element_number_list[-1][0] == atom.element:
            element_number_list[-1][1] += 1
        else:
            element_number_list.append([atom.element, 1])
    return element_number_list



from crystally.core.file_handling import *
from crystally.core.lattice import *
from crystally import examples
from crystally.core.vectors import *
from crystally.core import constants

import crystally.core.vasp_tools as vasp_tools

__all__ = ('Lattice', 'Atom', 'FractionalVector', 'write_poscar', 'read_poscar', 'examples', 'vasp_tools')



import setuptools

setuptools.setup(name='crystally',
                 version='0.2.1',
                 description='Python module to model and analyze crystal structures',
                 url='https://git.rwth-aachen.de/john.arnold/Crystally.git',
                 author='John P. Arnold',
                 author_email='arnold@pc.rwth-aachen.de',
                 license='None',
                 packages=setuptools.find_packages(),
                 package_dir={'crystally': 'crystally'},
                 install_requires=['numpy'])



from crystally.core.lattice import *
import numpy as np
from crystally.core.vasp_tools import get_element_number_list


__all__ = ("write_poscar", "read_poscar",)


def write_poscar(lattice: Lattice, file: str, poscar_description: str):
    """ Write a lattice to a file in the vasp POSCAR format.

    :param lattice: a :class:`~crystally.core.lattice.Lattice` object
    :param file: the file name
    :param poscar_description: the description that is written to the head of the file
    :return: None
    """
    header = "{:s} \n" \
             "1.00 \n" \
             "{:19.16f} {:19.16f} {:19.16f} \n" \
             "{:19.16f} {:19.16f} {:19.16f} \n" \
             "{:19.16f} {:19.16f} {:19.16f} \n" \
            .format(poscar_description, *tuple(lattice.vectors.flatten()))

    element_numbers = get_element_number_list(lattice)
    elements = [x[0] for x in element_numbers]
    numbers = [x[1] for x in element_numbers]
    for element in elements:
        header += "{} ".format(element)
    header += "\n"
    for number in numbers:
        header += "{} ".format(number)
    header += "\n"

    header += "Direct\n"

    with open(file, "w") as f:
        f.write(header)
        for atom in lattice:
            f.write("{:19.16f} {:19.16f} {:19.16f} \n".format(*tuple(atom.position.tolist())))


def read_poscar(file):
    """ Reads the lattice from a file in vasp POSCAR style.

    :param file: the file that should be read
    :return: a :class:`~crystally.core.lattice.Lattice` object
    """
    lattice = Lattice(atoms=[])

    with open(file, 'r') as f:
        f.readline()
        multiplier = float(f.readline())

        lattice_vectors = [np.array(list(map(float, f.readline().split()[:3])))*multiplier for _ in range(3)]

        lattice.vectors = lattice_vectors

        atom_names = f.readline().split()
        atom_numbers = list(map(int, f.readline().split()))
        element_list = [elem for i in range(len(atom_numbers)) for elem in [atom_names[i]]*atom_numbers[i]]

        option = f.readline()
        if option[0].lower() == "s":
            option = f.readline()

        for i in range(sum(atom_numbers)):
            position = np.array(list(map(float, f.readline().split()[:3])))
            if option[0].lower() == "c" or option[0].lower() == "k":
                position = position.dot(np.linalg.inv(lattice.vectors))
            lattice.atoms.append(Atom(element=element_list[i],
                                      position=position,
                                      sublattice=element_list[i]))
        return lattice
